package two

import (
	"bytes"
	"math"
	"sync"

	log "github.com/sirupsen/logrus"
)

func Solve(buf []byte) (int, error) {
	r := bytes.NewReader(buf)
	graph, err := parseLines(r)
	if err != nil {
		return 0, err
	}

	var wg sync.WaitGroup
	wg.Add(len(graph.antennae))

	for i, a := range graph.antennae {
		go func() {
			defer wg.Done()

			for j, b := range graph.antennae {
				if i == j || a.identifier != b.identifier {
					continue
				}

				all := []coords{a.coords, b.coords}
				for _, coords := range calcAntiNodePos(a.coords, b.coords, graph, all) {
					if !graph.antinodes.contains(coords) {
						graph.antinodes.insert(coords)
					}
				}
			}
		}()
	}

	wg.Wait()

	log.Debugf("\n%s\n", graph.debug())

	return graph.antinodes.len(), nil
}

func calcAntiNodePos(a, b coords, graph *graph, all []coords) []coords {
	xdiff := int(math.Abs(float64(a.x - b.x)))
	ydiff := int(math.Abs(float64(a.y - b.y)))

	var next coords
	if a.x < b.x && a.y < b.y {
		next = newCoords(b.x+xdiff, b.y+ydiff)
	} else if a.x < b.x && a.y > b.y {
		next = newCoords(b.x+xdiff, b.y-ydiff)
	} else if a.x > b.x && a.y < b.y {
		next = newCoords(b.x-xdiff, b.y+ydiff)
	} else {
		next = newCoords(b.x-xdiff, b.y-ydiff)
	}

	if graph.isOutOfBounds(next) {
		return all
	}
	return calcAntiNodePos(b, next, graph, append(all, next))
}
